@@ -26,7 +26,7 @@ from .exceptions import (
 )
 from .client import Fleet, SyncEnv
 from ._async.client import AsyncFleet, AsyncEnv
-from .models import InstanceResponse, Environment
+from .models import InstanceResponse, Environment, Run
 from .instance.models import Resource, ResetResponse
 
 # Import sync verifiers with explicit naming
@@ -46,11 +46,24 @@ from ._async.verifiers import (
     AsyncVerifierFunction,
 )
 
-# Import async tasks (default tasks are async for modern usage)  
-from ._async.tasks import Task, load_tasks as load_tasks_async
+# Import async tasks (default tasks are async for modern usage)
+from ._async.tasks import (
+    Task,
+    load_tasks as load_tasks_async,
+    load_tasks_from_file as load_tasks_from_file_async,
+    import_task as import_task_async,
+    import_tasks as import_tasks_async,
+    get_task as get_task_async,
+)
 
-# Import sync load_tasks function
-from .tasks import load_tasks
+# Import sync task functions
+from .tasks import (
+    load_tasks,
+    load_tasks_from_file,
+    import_task,
+    import_tasks,
+    get_task,
+)
 
 # Import shared types
 from .types import VerifierFunction
@@ -60,7 +73,7 @@ from . import env
 from . import global_client as _global_client
 from ._async import global_client as _async_global_client
 
-__version__ = "0.1.0"
+__version__ = "0.2.71"
 
 __all__ = [
     # Core classes
@@ -73,6 +86,7 @@ __all__ = [
     "SyncEnv",
     "Resource",
     "ResetResponse",
+    "Run",
     # Task models
     "Task",
     "VerifierFunction",
@@ -100,18 +114,24 @@ __all__ = [
     # Module-level functions (async is default)
     "load_tasks",
     "load_tasks_async",
+    "load_tasks_from_file",
+    "load_tasks_from_file_async",
+    "import_task",
+    "import_task_async",
+    "import_tasks",
+    "import_tasks_async",
+    "get_task",
+    "get_task_async",
     # Version
     "__version__",
 ]
 
 
-
-
 def configure(
     api_key: Optional[str] = None,
     base_url: Optional[str] = None,
-    max_retries: int | None = None,
-    timeout: float | None = None,
+    max_retries: Optional[int] = None,
+    timeout: Optional[float] = None,
 ):
     """Configure global clients (sync and async) once per process.
 
